@@ -3,7 +3,6 @@ package main
 import (
 	"context"
 	"math/rand"
-	"time"
 
 	"github.com/percona/percona-backup-mongodb/e2e-tests/pkg/tests/sharded"
 	"github.com/percona/percona-backup-mongodb/pbm/defs"
@@ -18,11 +17,11 @@ func runPhysical(t *sharded.Cluster, typ testTyp) {
 	}{
 		{"AWS", "/etc/pbm/aws.yaml"},
 		{"GCS", "/etc/pbm/gcs.yaml"},
+		{"GCS_HMAC", "/etc/pbm/gcs_hmac.yaml"},
 		{"Azure", "/etc/pbm/azure.yaml"},
 		{"FS", "/etc/pbm/fs.yaml"},
 	}
 
-	rand.Seed(time.Now().UnixNano())
 	rand.Shuffle(len(remoteStg), func(i, j int) {
 		remoteStg[i], remoteStg[j] = remoteStg[j], remoteStg[i]
 	})
