@@ -3,20 +3,22 @@ package pbm
 import (
 	"context"
 	"log"
+	"strings"
 
-	"github.com/docker/docker/api/types"
 	"github.com/docker/docker/api/types/container"
 	"github.com/docker/docker/api/types/filters"
 	"github.com/docker/docker/api/types/network"
-	docker "github.com/docker/docker/client"
+	"github.com/docker/docker/client"
 
 	"github.com/percona/percona-backup-mongodb/pbm/errors"
 )
 
 func ClockSkew(rsName, ts, dockerHost string) error {
-	log.Printf("== Skew the clock for %s on the replicaset %s ", ts, rsName)
+	if ts != "0" {
+		log.Printf("==Skew the clock for %s on the replicaset %s ", ts, rsName)
+	}
 
-	cn, err := docker.NewClientWithOpts(docker.WithHost(dockerHost))
+	cn, err := client.NewClientWithOpts(client.WithHost(dockerHost))
 	if err != nil {
 		return errors.Wrap(err, "docker client")
 	}
@@ -24,7 +26,7 @@ func ClockSkew(rsName, ts, dockerHost string) error {
 
 	fltr := filters.NewArgs()
 	fltr.Add("label", "com.percona.pbm.agent.rs="+rsName)
-	containers, err := cn.ContainerList(context.Background(), types.ContainerListOptions{
+	containers, err := cn.ContainerList(context.Background(), container.ListOptions{
 		Filters: fltr,
 	})
 	if err != nil {
@@ -37,18 +39,41 @@ func ClockSkew(rsName, ts, dockerHost string) error {
 			return errors.Wrapf(err, "ContainerInspect for %s", c.ID)
 		}
 
+		envs := append([]string{}, containerOld.Config.Env...)
+		if ts == "0" {
+			ldPreloadDefined := false
+			for _, env := range envs {
+				if strings.HasPrefix(env, "LD_PRELOAD") {
+					ldPreloadDefined = true
+					break
+				}
+			}
+
+			if !ldPreloadDefined {
+				log.Printf("Variable LD_PRELOAD isn't defined, skipping container restart for %s\n", containerOld.Name)
+				continue
+			}
+
+			var filteredEnvs []string
+			for _, env := range envs {
+				if !strings.HasPrefix(env, "LD_PRELOAD") {
+					filteredEnvs = append(filteredEnvs, env)
+				}
+			}
+			envs = filteredEnvs
+		} else {
+			envs = append(envs,
+				`LD_PRELOAD=/lib64/faketime/libfaketime.so.1`,
+				`FAKETIME=`+ts,
+			)
+		}
+
 		log.Printf("Removing container %s/%s\n", containerOld.ID, containerOld.Name)
-		err = cn.ContainerRemove(context.Background(), c.ID, types.ContainerRemoveOptions{Force: true})
+		err = cn.ContainerRemove(context.Background(), c.ID, container.RemoveOptions{Force: true})
 		if err != nil {
 			return errors.Wrapf(err, "remove container %s", c.ID)
 		}
 
-		envs := append([]string{}, containerOld.Config.Env...)
-		envs = append(envs,
-			`LD_PRELOAD=/lib64/faketime/libfaketime.so.1`,
-			`FAKETIME=`+ts,
-		)
-
 		log.Printf("Creating container %s/%s with the clock skew %s\n", containerOld.ID, containerOld.Name, ts)
 		containerNew, err := cn.ContainerCreate(context.Background(), &container.Config{
 			Image:  containerOld.Image,
@@ -67,7 +92,7 @@ func ClockSkew(rsName, ts, dockerHost string) error {
 			return errors.Wrap(err, "ContainerCreate")
 		}
 
-		err = cn.ContainerStart(context.Background(), containerNew.ID, types.ContainerStartOptions{})
+		err = cn.ContainerStart(context.Background(), containerNew.ID, container.StartOptions{})
 		if err != nil {
 			return errors.Wrap(err, "ContainerStart")
 		}
